package it.tref.liferay.mapit.map.configuration;

import it.tref.liferay.mapit.map.service.PluginPortletPreferencesLocalServiceUtil;

import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletConfig;
import javax.portlet.PortletPreferences;

import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.portlet.DefaultConfigurationAction;
import com.liferay.portal.kernel.portlet.PortletConfigurationLayoutUtil;
import com.liferay.portal.kernel.servlet.SessionErrors;
import com.liferay.portal.kernel.servlet.SessionMessages;
import com.liferay.portal.kernel.util.Constants;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.kernel.util.PropertiesParamUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.UnicodeProperties;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.kernel.util.WebKeys;
import com.liferay.portal.model.Layout;
import com.liferay.portal.model.PortletConstants;
import com.liferay.portal.security.permission.ActionKeys;
import com.liferay.portal.service.permission.PortletPermissionUtil;
import com.liferay.portal.theme.ThemeDisplay;
import com.liferay.portal.util.PortalUtil;
import com.liferay.portal.util.PortletKeys;
import com.liferay.portlet.PortletPreferencesFactoryUtil;

public class MapItMapConfigurationAction extends DefaultConfigurationAction {

  @Override
  public void processAction(PortletConfig portletConfig, ActionRequest actionRequest, ActionResponse actionResponse)
      throws Exception {

    String contextName = ParamUtil.getString(actionRequest, "contextName");

    if (Validator.isNull(contextName)) {
      super.processAction(portletConfig, actionRequest, actionResponse);
    } else {
      processAction(contextName, actionRequest, actionResponse);
    }
  }

  private void processAction(String contextName, ActionRequest actionRequest, ActionResponse actionResponse)
      throws Exception {

    String cmd = ParamUtil.getString(actionRequest, Constants.CMD);

    if (!cmd.equals(Constants.UPDATE)) {
      return;
    }

    ThemeDisplay themeDisplay = (ThemeDisplay) actionRequest.getAttribute(WebKeys.THEME_DISPLAY);

    UnicodeProperties properties = PropertiesParamUtil.getProperties(actionRequest, PREFERENCES_PREFIX);

    String portletResource = ParamUtil.getString(actionRequest, "portletResource");

    Layout layout = PortletConfigurationLayoutUtil.getLayout(themeDisplay);

    PortletPermissionUtil.check(themeDisplay.getPermissionChecker(), themeDisplay.getScopeGroupId(), layout,
        portletResource, ActionKeys.CONFIGURATION);

    long plid = themeDisplay.getPlid();
    String portletId = portletResource;

    long ownerId = PortletKeys.PREFS_OWNER_ID_DEFAULT;
    int ownerType = PortletKeys.PREFS_OWNER_TYPE_LAYOUT;

    if (PortletConstants.hasUserId(portletId)) {
      ownerId = PortletConstants.getUserId(portletId);
      ownerType = PortletKeys.PREFS_OWNER_TYPE_USER;
    }

    PortletPreferences portletPreferences = null;
    try {
      portletPreferences =
          PluginPortletPreferencesLocalServiceUtil.getPreferences(themeDisplay.getCompanyId(), ownerId, ownerType,
              plid, portletId, contextName);
    } catch (PortalException e) {
      portletPreferences = PortletPreferencesFactoryUtil.fromDefaultXML(StringPool.BLANK);
    }

    for (Map.Entry<String, String> entry : properties.entrySet()) {
      String name = entry.getKey();
      String value = entry.getValue();

      portletPreferences.setValue(name, value);
    }

    Map<String, String[]> portletPreferencesMap =
        (Map<String, String[]>) actionRequest.getAttribute(WebKeys.PORTLET_PREFERENCES_MAP);

    if (portletPreferencesMap != null) {
      for (Map.Entry<String, String[]> entry : portletPreferencesMap.entrySet()) {

        String name = entry.getKey();
        String[] values = entry.getValue();

        portletPreferences.setValues(name, values);
      }
    }

    if (SessionErrors.isEmpty(actionRequest)) {

      PluginPortletPreferencesLocalServiceUtil.updatePreferences(ownerId, ownerType, plid, portletId, contextName,
          portletPreferences);

      SessionMessages.add(actionRequest, PortalUtil.getPortletId(actionRequest)
          + SessionMessages.KEY_SUFFIX_REFRESH_PORTLET, portletResource);

      SessionMessages.add(actionRequest, PortalUtil.getPortletId(actionRequest)
          + SessionMessages.KEY_SUFFIX_UPDATED_CONFIGURATION);
    }
  }
}
