package it.tref.liferay.mapit.map.portlet;

import it.tref.liferay.mapit.constants.Constants;
import it.tref.liferay.mapit.map.service.PluginLocalServiceUtil;
import it.tref.liferay.mapit.map.util.MapItPluginUtil;
import it.tref.liferay.mapit.util.ExpandoUtil;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;

import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.service.ServiceContextFactory;
import com.liferay.util.bridges.mvc.MVCPortlet;

/**
 * Portlet implementation class MapItConfigurationPortlet
 */
public class MapItConfigurationPortlet extends MVCPortlet {

  private final static Log _log = LogFactoryUtil.getLog(MapItConfigurationPortlet.class);

  public void editCustomFields(ActionRequest actionRequest, ActionResponse actionResponse) throws IOException,
      PortletException {

    try {

      ServiceContext serviceContext = ServiceContextFactory.getInstance(actionRequest);

      String cmd = ParamUtil.getString(actionRequest, Constants.CMD);

      long classNameId = ParamUtil.getLong(actionRequest, "classNameId");

      if (cmd.equals(Constants.ADD)) {
        add(serviceContext.getCompanyId(), classNameId);
      } else if (cmd.equals(Constants.UPDATE)) {
        update(serviceContext.getCompanyId(), classNameId);
      } else if (cmd.equals(Constants.DELETE)) {
        delete(serviceContext.getCompanyId(), classNameId);
      }

    } catch (PortalException e) {
      _log.error(e, e);
      throw new PortletException(e);
    } catch (SystemException e) {
      _log.error(e, e);
      throw new PortletException(e);
    }
  }

  public void searchPlugins(ActionRequest actionRequest, ActionResponse actionResponse) throws IOException,
      PortletException {

    try {
      MapItPluginUtil.searchAndUpdatePlugins();
    } catch (SystemException e) {
      _log.error(e, e);
      throw new PortletException(e);
    }
  }

  public void activatePlugin(ActionRequest actionRequest, ActionResponse actionResponse) throws IOException,
      PortletException {

    try {

      long pluginId = ParamUtil.getLong(actionRequest, "pluginId");
      boolean active = ParamUtil.getBoolean(actionRequest, "active");

      try {
        if (!MapItPluginUtil.isMapItCompatiblePlugin(pluginId)) {
          active = false;
        }
      } catch (PortalException e) {
        _log.error(e, e);
        throw new PortletException(e);
      }

      PluginLocalServiceUtil.updatePlugin(pluginId, active);

    } catch (SystemException e) {
      _log.error(e, e);
      throw new PortletException(e);
    }
  }

  private void add(long companyId, long classNameId) throws SystemException {

    ExpandoUtil.addExpandoIfNotExists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT, false, false);
    ExpandoUtil.addExpandoIfNotExists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON, false, false);

    ExpandoUtil.addExpandoIfNotExists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT_HIDDEN, true, true);
    ExpandoUtil.addExpandoIfNotExists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON_HIDDEN, true, true);
  }

  private void delete(long companyId, long classNameId) throws SystemException {

    ExpandoUtil.deleteExpando(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT);
    ExpandoUtil.deleteExpando(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON);

    ExpandoUtil.deleteExpando(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT_HIDDEN);
    ExpandoUtil.deleteExpando(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON_HIDDEN);
  }

  private void update(long companyId, long classNameId) throws SystemException {

    // custom fields

    if (ExpandoUtil.exists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT)) {

      ExpandoUtil.addExpandoIfNotExists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT, false, false);
    }
    if (ExpandoUtil.exists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON)) {

      ExpandoUtil.addExpandoIfNotExists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON, false, false);
    }

    if (ExpandoUtil.exists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT_HIDDEN)) {

      ExpandoUtil.addExpandoIfNotExists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT_HIDDEN, true, true);
    }
    if (ExpandoUtil.exists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON_HIDDEN)) {

      ExpandoUtil.addExpandoIfNotExists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON_HIDDEN, true, true);
    }

    // type custom fields

    if (ExpandoUtil.isType(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT)) {

      ExpandoUtil.deleteExpando(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT);
      ExpandoUtil.addExpandoIfNotExists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT, false, false);
    }
    if (ExpandoUtil.isType(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON)) {

      ExpandoUtil.deleteExpando(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON);
      ExpandoUtil.addExpandoIfNotExists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON, false, false);
    }

    if (ExpandoUtil.isType(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT_HIDDEN)) {

      ExpandoUtil.deleteExpando(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT_HIDDEN);
      ExpandoUtil.addExpandoIfNotExists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT_HIDDEN, true, true);
    }
    if (ExpandoUtil.isType(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON_HIDDEN)) {

      ExpandoUtil.deleteExpando(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON_HIDDEN);
      ExpandoUtil.addExpandoIfNotExists(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON_HIDDEN, true, true);
    }

    // indexed custom fields

    if (ExpandoUtil.isIndexed(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT_HIDDEN)) {

      ExpandoUtil.updateExpando(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LAT_HIDDEN, true, true);
    }
    if (ExpandoUtil.isIndexed(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON_HIDDEN)) {

      ExpandoUtil.updateExpando(companyId, classNameId, Constants.EXPANDO_COLUMN_NAME_LON_HIDDEN, true, true);
    }

  }

}
